//GameManager.cpp

#include "Manager.h"
#include "GameManager.h"
#include "LogManager.h"
#include "WorldManager.h"
#include "GraphicsManager.h"
#include "Clock.h"
#include <iostream>
#include <unistd.h>
#include "InputManager.h"
#include "EventMouse.h"
#include "EventKeyboard.h"
#include "ResourceManager.h"

GameManager* GameManager::singletonInstance = NULL;
LogManager *logManager = LogManager::getInstance();
WorldManager *worldManager = WorldManager::getInstance();
GraphicsManager *graphicsManager = GraphicsManager::getInstance();
ResourceManager *resourceManager = ResourceManager::getInstance();
Clock *gameClock = new Clock();
Clock *testClock = new Clock();
long int elapsed = 33000;
int globalId = 0;
int event_list_count = 0;
string event[MAX_EVENTS];
GameObjectList obj_list[MAX_EVENTS];


GameManager::GameManager() 
{
	startUp();
}

GameManager::~GameManager() 
{
    
}

int GameManager::startUp(bool append, bool flush)
{
	return 1;
}

int GameManager::startUp(bool append, bool flush, time_t seed)
{
	return 1;
}

int GameManager::startUp()
{
	logManager->startUp(false, true);
	logManager->writeLog("Starting LogManager");
	graphicsManager->startUp();
	logManager->writeLog("Starting GraphicsManager");
	worldManager->startUp(false, true);
	logManager->writeLog("Starting WorldManager");
	InputManager *inputManager = InputManager::getInstance();
	inputManager->startUp();
	logManager->writeLog("Starting InputManager");
	
	ResourceManager *ResourceManager = ResourceManager::getInstance();
	ResourceManager->startUp();
	logManager->writeLog("Starting ResourceManager");
	Position world_corner(0,0);
	Box boundary(world_corner, graphicsManager->getHorizontal()-1,graphicsManager->getVertical()-1);
	worldManager->setBoundary(boundary);
	worldManager->setView(boundary);

	game_over = false;
	return 1;
}

void GameManager::run()
{
	
	while(game_over == false)
	{
	
		InputManager *inputManager = InputManager::getInstance();
		inputManager->getInput();
		
		worldManager->draw();
		
		//buffer new screen
		//logManager->writeLog("Tick");
		worldManager->update();
	
		
		if(DEFAULT_FRAME_TIME * 1000 - elapsed >= 0)
		{
			  
			usleep(DEFAULT_FRAME_TIME * 1000 - elapsed);
			
		}
		  
		elapsed = gameClock->delta();
		  
		graphicsManager->swapBuffers(); 
		  
	}
	setGameOver();

}

void GameManager::run(int frame_time)
{
	while(game_over == false)
	{

	}
}

void GameManager::setGameOver()
{
	game_over = true;
}

int GameManager::shutDown()
{
	logManager->writeLog("Closing WorldManager");
	worldManager->shutDown();
	logManager->writeLog("Closing GraphicsManager");
	graphicsManager->shutDown();
	logManager->writeLog("Closing InputManager");
	InputManager *inputManager = InputManager::getInstance();
	inputManager->shutDown();
	logManager->writeLog("Closing LogManager");
	logManager->shutDown();
	
	return 1;
}

int GameManager::makeId()
{
	return 1 + globalId;
}

GameManager* GameManager::getInstance()
{
	if (singletonInstance == NULL) 
	{
		singletonInstance = new GameManager();
	}
	return singletonInstance;
}


