//GameObject.cpp

//Includes
#include "Clock.h"
#include <time.h>
#include "LogManager.h"
#include "GameManager.h"
#include "GameObject.h"
#include "Position.h"
#include <iostream>
#include <unistd.h>
#include <string>
#include "WorldManager.h"
#include "GraphicsManager.h"
#include "Frame.h"
#include "Sprite.h"
#include "Box.h"
#include <stdio.h>
#include <math.h>
using namespace std;

 
GameObject::GameObject()
{
	altitude = 0;
	sprite_slowdown = 4;
	sprite_slowdown_count = 0;
	id = 0;
	is_solid = false;
	type = "Not_An_Object";
	pos = Position::Position(0,0); //May have some trouble
	WorldManager *game_world = WorldManager::getInstance();
	game_world->insertObj(this);
	Box box;
	
}

GameObject::~GameObject()
{
	WorldManager *game_world = WorldManager::getInstance();
	game_world->removeObj(this);
}

void GameObject::setType(string new_type)
{
	new_type = type;
}

string GameObject::getType()
{
	return type;
}

void GameObject::setPos(Position new_pos)
{
	pos = new_pos;
}

Position GameObject::getPos()
{
	return pos;
}

void GameObject::setId(int new_id)
{
	id = new_id;
}

int GameObject::getId()
{
	return id;
}

int GameObject::eventHandler(Event *e)
{
	return 1;
}

void GameObject::draw()
{
	if(!p_sprite)
	{
		return;
	}
	
	GraphicsManager *graph_mgr = GraphicsManager::getInstance();
	graph_mgr->drawFrame(pos, p_sprite->getFrame(getSpriteIndex()), true, p_sprite->getColor());
	
	if(getSpriteIndex() == p_sprite->getFrameCount())
	{
		setSpriteIndex(0);
	}
	
	
	if(getSpriteSlowdown() == 0)
	{
		return;
	}

	int count = getSpriteSlowdownCount()+1; 

	if(count == getSpriteSlowdown())
	{
		int next = getSpriteIndex() + 1;
		setSpriteIndex(next);
		setSpriteSlowdownCount(0);
	}
	else 
	{
		setSpriteSlowdownCount(count);
	}
}

void GameObject::setSolid(bool solid)
{
	is_solid = solid;
}

bool GameObject::isSolid()
{
	return is_solid;
}

 void GameObject::setAltitude(int new_altitude)
 {
	 altitude = new_altitude;
 }
 int GameObject::getAltitude()
 {
	 return altitude;
 }

  bool GameObject::isCentered()
  {
	  return sprite_center;
  }
  void GameObject::setCentered(bool centered)
  {
	  sprite_center = centered;
  }
  Sprite* GameObject::getSprite()
  {
	  return p_sprite;
  }
  void GameObject::setSprite(Sprite *p_new_sprite)
  {
	  pos = Position::Position(0,0); //May have some trouble
	  p_sprite = p_new_sprite;
	  p_sprite->setWidth(getSprite()->getFrame(0).getWidth());
	  p_sprite->setHeight(getSprite()->getFrame(0).getHeight());
	  box.setHorizontal(p_sprite->getWidth());
	  box.setVertical(p_sprite->getHeight());
	  box.setCorner(pos);
  }
  int GameObject::getSpriteIndex()
  {
	  return sprite_index;
  }
  void GameObject::setSpriteIndex(int new_sprite_index)
  {
	  sprite_index = new_sprite_index;
  }
  int GameObject::getSpriteSlowdown()
  {
	  return sprite_slowdown;
  }
  void GameObject::setSpriteSlowdown(int new_sprite_slowdown)
  {
	  sprite_slowdown = new_sprite_slowdown;
  }
  void GameObject::setSpriteSlowdownCount(int new_sprite_slowdown_count)
  {
	 sprite_slowdown_count = new_sprite_slowdown_count;
  }
  int GameObject::getSpriteSlowdownCount()
  {
	  return sprite_slowdown_count; 
  }
  void GameObject::setBox(Box new_box)
  {
	  box = new_box;
  }
  Box GameObject::getBox()
  {
	return box;
  }
  void GameObject::setXVelocity(float new_x_velocity)
  {
	  x_velocity = new_x_velocity;
  }
  void GameObject::setYVelocity(float new_y_velocity)
  {
	   y_velocity = new_y_velocity;
  }
  float GameObject::getXVelocity()
  {
	  return x_velocity;
  }
  float GameObject::getYVelocity()
  {
	  return y_velocity;
  }
  int GameObject::getXVelocityStep()
  {
	  // see if there is an x-velocity 
	  if (!x_velocity)
	  {
		return 0;
	  }
	// see if time to move
	x_velocity_countdown--;

	if (x_velocity_countdown > 0)
	{
		 return 0;
	}
	// ok, time to move, so figure out how far
	int step = 0;

	do 
	{
		x_velocity_countdown += fabs(1/x_velocity);
		(x_velocity < 0) ? step-- : step++;
	} while (x_velocity_countdown <= 0);
		
	return step;

  }
  int GameObject::getYVelocityStep()
  {
	   // see if there is an x-velocity 
	  if (!y_velocity)
	  {
		return 0;
	  }
	// see if time to move
	y_velocity_countdown--;

	if (y_velocity_countdown > 0)
	{
		 return 0;
	}
	// ok, time to move, so figure out how far
	int step = 0;

	do 
	{
		y_velocity_countdown += fabs(1/y_velocity);
		(y_velocity < 0) ? step-- : step++;
	} while (y_velocity_countdown <= 0);

	return step;

  }

