//
// GameObject.h
//

#ifndef __GAME_OBJECT_H__
#define __GAME_OBJECT_H__

#include "Position.h"		// objects need a location
#include <string>
#include "Event.h"
#include "Sprite.h"
#include "Box.h"

using namespace std;

class GameObject {

 private:
  int id;	

 protected:
  string type;		      // user-defined identification
  Position pos;			// position in the game world
  bool is_solid;
  int altitude;
  bool sprite_center;
  Sprite* p_sprite;
  int sprite_index;
  int sprite_slowdown;
  int sprite_slowdown_count;
  Box box;
  float x_velocity;
  float x_velocity_countdown;
  float y_velocity;
  float y_velocity_countdown;
  
 public:
  GameObject();
  virtual ~GameObject();
  virtual void draw();
  void setId(int new_id);
  int getId();
  void setType(string new_type);
  string getType();
  void setPos(Position new_pos);
  Position getPos();
  void setSolid(bool solid);
  virtual bool isSolid();
  void setAltitude(int new_altitude);
  int getAltitude();
  bool isCentered();
  void setCentered(bool centered);
  Sprite* getSprite();
  void setSprite(Sprite *p_new_sprite);
  virtual int eventHandler(Event *e);
  int getSpriteIndex();
  void setSpriteIndex(int new_sprite_index);
  void setSpriteSlowdown(int new_sprite_slowdown);
  int getSpriteSlowdown();
  void setSpriteSlowdownCount(int new_sprite_slowdown_count);
  int getSpriteSlowdownCount();
  void setBox(Box new_box);
  Box getBox();
  void setXVelocity(float new_x_velocity);
  void setYVelocity(float new_y_velocity);
  float getXVelocity();
  float getYVelocity();
  int getXVelocityStep();
  int getYVelocityStep();

};
 
#endif // __GAME_OBJECT_H__

