//GraphicsManager.cpp

#include "Manager.h"
#include "GraphicsManager.h"
#include <curses.h>
#include "GraphicsManager.h"
#include "LogManager.h"
#include <string>
#include "Box.h"
#include "WorldManager.h"
#include "Position.h"

using namespace std;

int max_y, max_x;

GraphicsManager* GraphicsManager::singletonInstance = NULL;
bool is_started;
int COLOR_DEFAULT = 1;

GraphicsManager::GraphicsManager() 
{
	is_started = false;
}

GraphicsManager::~GraphicsManager() 
{
   
}

int GraphicsManager::startUp()
{
	initscr();
	getmaxyx(stdscr, max_y, max_x);
	curr_win = win1;
	win1 = newwin(0,0,0,0);
	win2 = newwin(0,0,0,0);
	wattron(curr_win, A_BOLD);
	leaveok(curr_win, TRUE);

	if (has_colors() == TRUE)
	{
		LogManager *logManager = LogManager::getInstance();
		logManager->writeLog("Color Enabled");
		start_color();
		init_pair(1,COLOR_GREEN,COLOR_BLACK);
		init_pair(2,COLOR_RED,COLOR_BLACK);
		init_pair(3,COLOR_CYAN,COLOR_BLACK);
		init_pair(4,COLOR_WHITE,COLOR_BLACK);
	}
	else
	{
		LogManager *logManager = LogManager::getInstance();
		logManager->writeLog("No Color Enabled");
	}
	is_started = true;
	return 1;
}

void GraphicsManager::shutDown()
{
	endwin();
	is_started = false;
}


int GraphicsManager::swapBuffers()
{
	if(curr_win == win1)
	{
		wrefresh(curr_win);
		werase(win2);
		curr_win = win2;
		
	}
	else
	{
		wrefresh(curr_win);
		werase(win1);
		curr_win = win1;
		
	}
	return 0;
}

int GraphicsManager::drawCh(Position world_pos, char ch, int color)
{
	Position screen_pos = worldToScreen(world_pos);
	wattron(curr_win, COLOR_PAIR(color));
	mvwaddch(curr_win, screen_pos.getY(), screen_pos.getX(), ch);
	wattroff(curr_win, COLOR_PAIR(color));
	return 0;
}

int GraphicsManager::drawString(Position world_pos, const char *str, int color)
{
	Position screen_pos = worldToScreen(world_pos);
	wattron(curr_win, COLOR_PAIR(color));
	mvwaddstr(curr_win, screen_pos.getY(), screen_pos.getX(), str);
	wattroff(curr_win, COLOR_PAIR(color));
	return 0;
}


int GraphicsManager::getHorizontal()
{
	return max_x;
}
  
int GraphicsManager::getVertical()
{
	return max_y;
}

GraphicsManager* GraphicsManager::getInstance()
{
	if (singletonInstance == NULL) 
	{
		singletonInstance = new GraphicsManager();
	}
	return singletonInstance;
}

bool GraphicsManager::isStarted()
{
	return is_started;
}

int GraphicsManager::drawFrame(Position world_pos, Frame frame, bool centered, int color)
{
	int y_offset;
	int x_offset;
	

	world_pos = worldToScreen(world_pos);

	if(frame.getString().length() == 0)
	{
		LogManager *logManager = LogManager::getInstance();
		logManager->writeLog("Empty Frame");
		return -1;
	}
	if(centered)
	{
		y_offset = frame.getHeight() / 2;
		x_offset = frame.getWidth() / 2;
	}
	else
	{
		return 0;
	}

	string str = frame.getString();

	for(int y = 0; y < frame.getHeight(); y++)
	{
		for(int x = 0; x < frame.getWidth(); x++)
		{
			Position temp_pos(world_pos.getX() - x_offset + x, world_pos.getY()-y_offset+y);
			drawCh(temp_pos, str[y*frame.getWidth() + x], color);
		}
	}
	return 1;
}

Position GraphicsManager::worldToScreen(Position world_pos)
{
	WorldManager *game_world = WorldManager::getInstance();
	int view_x = game_world->getView().getCorner().getX();
	int view_y = game_world->getView().getCorner().getY();
	Position screen_pos(world_pos.getX() - view_x, 	world_pos.getY() - view_y);
	return screen_pos;
}




