// GraphicsManager.h

#ifndef __GRAPHICS_MANAGER_H__
#define __GRAPHICS_MANAGER_H__

#ifdef CYGWIN
#include <ncurses/curses.h>
#else
#include <curses.h>
#endif

#include "Position.h"
#include "Manager.h"
#include "Frame.h"
#include "Box.h"
#include <string>
using namespace std;


// the graphics manager is a singleton instance
class GraphicsManager : public Manager {

 private:
  GraphicsManager (GraphicsManager const&); // don't allow copy
  void operator=(GraphicsManager const&);   // don't allow assignment
  GraphicsManager();		// private since a singleton
  static GraphicsManager* singletonInstance;
  
 protected:
  WINDOW *win1, *win2;		// two window buffers for drawing
  WINDOW *curr_win;		// current buffer
  int window_horiz;		// max window width
  int window_vert;		// max window height

 public:
  ~GraphicsManager();
  static GraphicsManager* getInstance();
  Position worldToScreen(Position world_pos);
  int startUp();
  void shutDown();
  int swapBuffers();
  int drawCh(Position world_pos, char ch, int color);
  int drawString(Position world_pos, const char * str, int color);
  int getHorizontal();
  int getVertical();
  bool isStarted();
  int drawFrame(Position world_pos, Frame frame, bool centered, int color);

};

#endif //__GRAPHICS_MANAGER_H__

