// Hero.cpp

//Includes
#include "Clock.h"
#include <time.h>
#include "GameManager.h"
#include "GameObject.h"
#include "Position.h"
#include <iostream>
#include <unistd.h>
#include <string>
#include "WorldManager.h"
#include "Hero.h"
#include "Event.h"
#include "EventStep.h"
#include "EventKeyboard.h"
#include "EventMouse.h"
#include "LogManager.h"
#include "Manager.h"
#include "GraphicsManager.h"
#include <stdlib.h>
#include "EventOut.h"
#include "EventCollision.h"
#include "InputManager.h"
#include "ResourceManager.h"
#include "Gui.h"
#include <sstream>
#include "Straight.h"
#include "BaseEnemy.h"
using namespace std;

Position newPos = Position::Position(0,0);
Position pos = Position::Position(0,0);
bool backUp;
int scoreTimer = 15; // how long the score text should be on the screen
int hitTimer = 0;
int singleScore; //single score of charge
int health; //amt of health
Sprite *p_temp_sprite;
Position setpos; //position of score text
int printInstructionsTimer = 600; //timer for instructions to be pritned to screen
int gameOverTimer = 300; //timer for game Over Art
bool shipDead = false;

Hero::Hero() 
{
	//set up Id, position, location, and sprite info
	GameManager *gameManager = GameManager::getInstance();
	id = gameManager->makeId();
	pos.setX(0);
    pos.setY(0);
	setType("Hero");
	is_solid = true;
	state = "stationary";
	Manager manager = Manager::Manager();
	manager.registerInterest(this, STEP_EVENT);
	InputManager *inputManager = InputManager::getInstance();
	inputManager->registerInterest(this, KEYBOARD_EVENT);
	inputManager->registerInterest(this, MOUSE_EVENT);
	ResourceManager *resource_manager = ResourceManager::getInstance();
	resource_manager->loadSprite("saucer-spr.txt", "Hero");
	setSpriteSlowdown(10);
	setSpriteSlowdownCount(0);
	scoreTimer = 0;
	hitTimer = 0;
	health = 100;
	score = 0;
	combo = 1;
	singleScore = 250 + (250*combo);
	printInstructionsTimer = 600;
	gameOverTimer = 0; // Time until gameover sprite goes away
	altitude = 0;

  	// Sprite data
  	p_temp_sprite = resource_manager->getSprite("Hero");
  	GameObject::setSprite(p_temp_sprite);

	Position new_pos(5, 10);
	WorldManager *world_manager = WorldManager::getInstance();
	world_manager->moveObj(this, new_pos);

}

Hero::~Hero()
{
    Manager manager = Manager::Manager();
	manager.unregisterInterest(this, STEP_EVENT);
	InputManager *inputManager = InputManager::getInstance();
	inputManager->unregisterInterest(this, KEYBOARD_EVENT);
	inputManager->unregisterInterest(this, MOUSE_EVENT);
}

// handle event
// return 0 if ignored, else 1
int Hero::eventHandler(Event *e) 
{
GraphicsManager *graphicsManager = GraphicsManager::getInstance(); 
  if (e->getType() == STEP_EVENT) 
  {
	 if(shipDead == true) //Part of game over graphic
	 {
			graphicsManager->drawString(Position(0,0), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,1), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,2), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,3), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,4), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,5), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,6), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,7), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,8), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,9), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,10), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,11), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,12), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,13), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,14), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,15), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,16), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,17), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,18), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,19), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,20), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,21), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,22), "********************************************************************************", 1);
			graphicsManager->drawString(Position(0,23), "********************************************************************************", 1);
	 }
	  
	 singleScore = 250 + (250*combo);
	//dynamic GUI elements
	string scoreStr = "Score: ";
	string s;
	stringstream out; 
	out << score; 
	s = out.str(); 
	scoreStr += s;
	const char* temp = scoreStr.c_str();
	graphicsManager->drawString(Position(1,0), temp, 1);

	string healthStr = "Health: ";
	string s1;
	stringstream out1; 
	out1 << health; 
	s1 = out1.str(); 
	healthStr += s1;
	const char* temp1 = healthStr.c_str();
	graphicsManager->drawString(Position(68,0), temp1, 1);

	if(scoreTimer > 0)
	{
		//post score text for 27 ticks
		if(scoreTimer == 27)
		{
			setpos = Position(pos.getX(), pos.getY() - 3);
		}
		Gui *gui = Gui::getInstance();
		gui->printScore(singleScore-250,combo-1,setpos);
		scoreTimer --;
	}

	if(hitTimer > 0)
	{
		p_temp_sprite->setColor(4);
		hitTimer--;
	}

	//blink the game over sprite
	if(gameOverTimer > 0)
	{
		if(gameOverTimer > 400)
		{
			printGameOver();
		}
		if(gameOverTimer > 200 && gameOverTimer < 350 )
		{
			printGameOver();
		}
		if(gameOverTimer > 0 && gameOverTimer < 150)
		{
			printGameOver();
		}
		gameOverTimer--;

		if(gameOverTimer == 0)
		{
			WorldManager *world_manager = WorldManager::getInstance();
			world_manager->markForDelete(this);
			GameManager *gameManager = GameManager::getInstance();
			gameManager->setGameOver();
		}
	}

	//Print gameplay Instructions
	if(printInstructionsTimer > 0)
	{
		printInstructions(1);
		if(printInstructionsTimer < 450)
		{
			printInstructions(2);
		}
		if(printInstructionsTimer < 300)
		{
			printInstructions(3);
		}
		if(printInstructionsTimer < 150)
		{
			printInstructions(4);
		}
		printInstructionsTimer--;
	}
	
	if(state == "stationary" && hitTimer == 0) 
	{
		p_temp_sprite->setColor(1);
	}

	if(state == "charge")
	{
		string tail = "*********";
		p_temp_sprite->setColor(2);
		graphicsManager->drawString(Position(pos.getX()-15, pos.getY()), tail.c_str(), 4);
		graphicsManager->drawString(Position(pos.getX()-15, pos.getY()-1),tail.c_str(), 4);
		
	}
	if(state == "retreat" && hitTimer == 0)
	{
		p_temp_sprite->setColor(3);
	}

	//if at end of screen, go to cooldown state
	if(pos.getX() >= 74)
	{
		x_velocity = -.3 ;
		state = "retreat";
	}
	//reset position
	if(pos.getX() <= 4)
	{
		x_velocity = 0;
		Position new_pos(5, pos.getY());
		WorldManager *world_manager = WorldManager::getInstance();
		world_manager->moveObj(this, new_pos);
		state = "stationary";
		combo = 1;
	}
	
	return 1;
  }
  if (e->getType() == KEYBOARD_EVENT) 
  {
    	LogManager *logManager = LogManager::getInstance();
		logManager->writeLog("KEYBOARD EVENT");

	if(state == "stationary")
	{
		EventKeyboard *p_keyboard_event = static_cast <EventKeyboard *> (e);
		
		switch(p_keyboard_event->getKey()) 
		{
		case KEY_UP:			// up arrow
			move(-1);
			break;
		case KEY_DOWN:		// down arrow
			move(+1);
			break;
		case 32:		//space bar
			charge();
	break;
		};
	}
	if(state == "retreat") //disallow chargin while on cooldown
	{
		EventKeyboard *p_keyboard_event = static_cast <EventKeyboard *> (e);
		
		switch(p_keyboard_event->getKey()) 
		{
		case KEY_UP:			// up arrow
			move(-1);
			break;
		case KEY_DOWN:		// down arrow
			move(+1);
			break;
		};
	}
	return 1;
  } 
  if (e->getType() == MOUSE_EVENT)  //for testing purposes
  {
    LogManager *logManager = LogManager::getInstance();
	logManager->writeLog("MOUSE EVENT");
	WorldManager *world_manager = WorldManager::getInstance();
	GameObjectList myList = world_manager->getAllObjects();

	// Update object positions based on their velocities
	 GameObjectListIterator vi(&myList);
     vi.first();
	 while(vi.isDone() == false)
     {
		 LogManager *logManager = LogManager::getInstance();
		 logManager->writeLog("%d", vi.currentObj()->getBox().getCorner().getX());
		 logManager->writeLog("%d", vi.currentObj()->getBox().getCorner().getY());
		 vi.next();
     }
		
	return 1;
  }

  if (e->getType() == OUT_EVENT) // cannot really go out of bounds 
  {
    LogManager *logManager = LogManager::getInstance();
	logManager->writeLog("OUT EVENT");
	
	return 1;
  } 

  if (e->getType() == COLLISION_EVENT) 
  {
    LogManager *logManager = LogManager::getInstance();
	logManager->writeLog("COLLISION EVENT");
	EventCollision *p_collision_event = static_cast <EventCollision *> (e);
	WorldManager *world_manager = WorldManager::getInstance();

if(state == "stationary" || state == "retreat")
{	
	    if(health >= 10)
	    {	
			setHealth(health - 10); //decrement health on hit
			hitTimer = 13;
			if(health == 0 && gameOverTimer == 0)
			{
				shipDead = true;
				gameOverTimer = 500;
				InputManager *inputManager = InputManager::getInstance();
				inputManager->unregisterInterest(this, KEYBOARD_EVENT);
			}

		}
		else if(gameOverTimer == 0)
		{
			shipDead = true;
			gameOverTimer = 500;
			InputManager *inputManager = InputManager::getInstance();
			inputManager->unregisterInterest(this, KEYBOARD_EVENT);
			health = 0;
			hitTimer = 10;
		}
}

if(state == "charge") //add score to screen
{
		scoreTimer = 27;
		combo++;
		score = score + singleScore;  
		world_manager->markForDelete(p_collision_event -> getObj1());
	  
}

}

	return 1;
   } 


void Hero::draw()
{
  GameObject::draw();
}

void Hero::setId(int new_id)
{
	id = new_id;
}

int Hero::getId()
{
	return id;
}

bool Hero::isSolid()
{
	return is_solid;
}

string Hero::getState()
{
	return state;
}
 
void Hero::setState(string new_state)
{
	state = new_state;
}

int Hero::move(int dy)
{

	WorldManager *world_manager = WorldManager::getInstance();
	Position new_pos(pos.getX(), pos.getY() + dy);

  // if stays on screen, allow move
  if ((new_pos.getY() > 0) && 
      (new_pos.getY() < world_manager->getBoundary().getVertical()))
  {
    world_manager->moveObj(this, new_pos);
  }
	
  return 0;
}

int Hero::charge()
{
	state = "charge";
	x_velocity = 2;
	return 1;
}

void Hero::setHealth(int new_health)
{
	health = new_health;
}

int Hero::getHealth()
{
	return health;
}

void Hero::setScore(int new_score)
{
	score = new_score;
}

int Hero::getScore()
{
	return score;
}
  
void Hero::setCombo(int new_combo)
{
	combo = new_combo;
}

int Hero::getCombo()
{
	return combo;
}

void Hero::printInstructions(int line) // print instructions
{
	GraphicsManager *graphicsManager = GraphicsManager::getInstance();
	graphicsManager->drawString(Position(21,9), "---------------------------------------------", 4);
	graphicsManager->drawString(Position(21,14), "---------------------------------------------", 4);

	if(line == 1)
	{
		graphicsManager->drawString(Position(21,10), "Control your ship with the up/down arrow keys", 1);
	}
	if(line == 2)
	{
		graphicsManager->drawString(Position(24,11), "Press space to charge into enemy ships", 1);
	}
	if(line == 3)
	{
		graphicsManager->drawString(Position(26,12), "Avoid all enemies when not charging", 1);
	}
	if(line == 4)
	{
		graphicsManager->drawString(Position(38,13), "Good Luck!", 1);
	}

}

void Hero::printGameOver() // print game over graphic
{
	GraphicsManager *graphicsManager = GraphicsManager::getInstance();
	graphicsManager->drawString(Position(15,9), "   ____                         ___                 ", 3);
	graphicsManager->drawString(Position(15,10), "  / ___| __ _ _ __ ___   ___   / _ \\__   _____ _ __ ", 4);
	graphicsManager->drawString(Position(15,11), " | |  _ / _` | '_ ` _ \\ / _ \\ | | | \\ \\ / / _ \\ '__|", 3);
	graphicsManager->drawString(Position(15,12), " | |_| | (_| | | | | | |  __/ | |_| |\\ V /  __/ |   ", 4);
	graphicsManager->drawString(Position(15,13), "  \\____|\\__,_|_| |_| |_|\\___|  \\___/  \\_/ \\___|_|   ", 3);
}















		