//InputManager.cpp

#include "Manager.h"
#include "InputManager.h"
#include <curses.h>
#include "GraphicsManager.h"
#include "LogManager.h"
#include "Event.h"
#include "EventKeyboard.h"
#include "EventMouse.h"
#include "GameManager.h"

InputManager* InputManager::singletonInstance = NULL;
extern string event[MAX_EVENTS];
extern GameObjectList obj_list[MAX_EVENTS];
extern int event_list_count;


InputManager::InputManager() 
{
	is_started = false;
}

InputManager::~InputManager() 
{
    
}

int InputManager::startUp()
{
	GraphicsManager *graphicsManager = GraphicsManager::getInstance();
	if(graphicsManager->isStarted() == false)
	{
		return -1;
	}
	keypad(stdscr, TRUE);
	cbreak();
	nonl();
	curs_set(0);
	nodelay(stdscr, TRUE);
	mmask_t mask;
	mask = BUTTON1_CLICKED | BUTTON2_CLICKED | BUTTON1_DOUBLE_CLICKED | BUTTON2_DOUBLE_CLICKED;
	mousemask(mask, NULL);

	is_started = true;

	return 0;
}

void InputManager::shutDown()
{
	curs_set(1);
	is_started = false;
}

bool InputManager::isValid(string event_name)
{
	if(event_name == KEYBOARD_EVENT || event_name == MOUSE_EVENT)
	{
		return true;
	}
	return false;
}

void InputManager::getInput()
{
	MEVENT m_event;
	int c = getch();

	if(c != ERR)
	{
		if (c==KEY_MOUSE && getmouse(&m_event) == OK) 
		{
			if (m_event.bstate & BUTTON1_CLICKED) 
			{
			  	EventMouse mouse;
				mouse.setMouseX(m_event.x);
				mouse.setMouseY(m_event.y);
				if(m_event.bstate == LEFT_BUTTON_CLICK)
				{
					mouse.setMouseAction(LEFT_BUTTON_CLICK);
				}
				if(m_event.bstate == LEFT_BUTTON_DOUBLECLICK)
				{
					mouse.setMouseAction(LEFT_BUTTON_DOUBLECLICK);
				}
				if(m_event.bstate == RIGHT_BUTTON_CLICK)
				{
					mouse.setMouseAction(RIGHT_BUTTON_CLICK);
				}
				if(m_event.bstate == RIGHT_BUTTON_DOUBLECLICK)
				{
					mouse.setMouseAction(RIGHT_BUTTON_DOUBLECLICK);
				}
				Manager manager = Manager::Manager();
				manager.onEvent(&mouse);
			}
			else
			{
			
			}
		}
		else
		{
			 Manager manager = Manager::Manager();
			 EventKeyboard keyboard;
			 keyboard.setKey(c);
			 manager.onEvent(&keyboard);
			 LogManager *logManager = LogManager::getInstance();
			 logManager->writeLog("Pressed: %c", c);
		}
	}
}

InputManager* InputManager::getInstance()
{
	if (singletonInstance == NULL) 
	{
		singletonInstance = new InputManager();
	}
	return singletonInstance;
}

bool InputManager::isStarted()
{
	return is_started;
}

int InputManager::registerInterest(GameObject *p_go, string event_name)
{
		if(isValid(event_name) == true)
		{
			for(int i = 0; i <= event_list_count; i++)
			{
			
				if(event[i] == event_name)
				{
					obj_list[i].insert(p_go);
					return 0;
				}
			}
			if(event_list_count < MAX_EVENTS)
			{
				 event[event_list_count] = event_name;
				 obj_list[event_list_count].insert(p_go);
				 event_list_count = event_list_count + 1;
			}
			
			return 0;
		}
    return -1;
}
int InputManager::unregisterInterest(GameObject *p_go, string event_name)
{
	for(int i = 0; i < event_list_count; i++)
	{
		if(event_name == event[i])
		{
			GameObjectListIterator goli(&obj_list[i]);
			goli.first();
	 
			while(goli.isDone() == false)
			{
				if(p_go == goli.currentObj())
				{
					obj_list[i].remove(goli.currentObj());
					LogManager *logManager = LogManager::getInstance();
					logManager->writeLog("Unregistering Input Event");
					return 0;
				}
				goli.next();
			}
		}
	}
	return -1;
}
