//Straight.cpp

#include "Straight.h"
#include "ResourceManager.h"
#include "WorldManager.h"
#include "LogManager.h"
#include "GameManager.h"
#include "EventStep.h"
#include "EventCollision.h"
#include "EventOut.h"
#include "Manager.h"
#include "Hero.h"
#include "GraphicsManager.h"
#include <stdlib.h>

bool is_solid;

Straight::Straight(){
	//Sprite and interest data
	
	ResourceManager *rm = ResourceManager::getInstance();
	rm->loadSprite("straight-spr.txt", "straight");

	Manager manager = Manager::Manager();
	manager.registerInterest(this, STEP_EVENT);

	Sprite* p = rm->getSprite("straight");
	GameObject::setSprite(p);
	setSpriteSlowdown(0);

	//set X pos 20 steps before start of screen to stagger time
	WorldManager *wm = WorldManager::getInstance();
	int w_horiz = wm->getBoundary().getHorizontal() + random() % 100;
	int w_vert = wm->getBoundary().getVertical();
	pos.setX(w_horiz + 50);
	
	randMove = random()%3; //adds staggered enemies
	
	if(randMove == 2) //If rand move = 2, make that enemy stagger
	{
		y_velocity = -.35;
		pos.setY(9);
		
	}
	else // normal straight movement
	{
		int randomY = random() % w_vert + 4; // random y Pos
		
		if(randomY < wm->getBoundary().getVertical())
		{
			pos.setY(9);
		}
		pos.setY(randomY);
		
		p->setColor(4);
	}

	
	
	x_velocity = -0.15;

	setType("Enemy");

	

	is_solid = true;

	altitude = 1;
}

Straight::~Straight(){
	WorldManager *world_manager = WorldManager::getInstance();
	Manager manager = Manager::Manager();
	manager.unregisterInterest(this, STEP_EVENT);
	world_manager->unregisterInterest(this, OUT_EVENT);
}

void Straight::out(){
	if(getPos().getX() >= 0){
		return;
	}
		//respawn enemy with slightly faster speed
		WorldManager *wm = WorldManager::getInstance();
		//x_velocity += -.02;
		int w_horiz = wm->getBoundary().getHorizontal();
		pos.setX(w_horiz+ random()% 100 + 50);
}

int Straight::eventHandler(Event *e){

	if(e->getType() == OUT_EVENT){
		out();
		return 1;
	}

	if(e->getType() == COLLISION_EVENT){
		
		LogManager *logManager = LogManager::getInstance();
		logManager->writeLog("ENEMY IS HIT");
		
		//respawn enemy with slightly faster speed
		x_velocity += -.02;
		WorldManager *wm = WorldManager::getInstance();
		int w_horiz = wm->getBoundary().getHorizontal();
		pos.setX(w_horiz+ random()% 100 + 50);

		return 1;
	}

	if(e->getType() == STEP_EVENT)
	{	
		WorldManager *world_manager = WorldManager::getInstance();
		
		if(randMove != 2) 
		{
			if (e->getType() == STEP_EVENT) //set enemies so they do not get cut off
			{
				if(pos.getY() <= 2)
				{
					pos.setY(3);
				}
				if(pos.getY() >= world_manager->getBoundary().getVertical() -2 )
				{
					pos.setY(world_manager->getBoundary().getVertical() - 3);
				}
			}
		}
		else if(randMove == 2) //Change Y velocity dependaing on top or bottom of screen
		{
			if(pos.getY() < 3)
			{
				y_velocity = 0.35;
			}
			else if(pos.getY() > 18)
			{
				y_velocity = -0.35;		
			}
		}
	}

	return 0;
}

void Straight::collision(EventCollision *p_c){
	
}

bool Straight::isSolid()
{
	return is_solid;
}

