//WorldManager.cpp

//Includes
#include "Clock.h"
#include <time.h>
#include "LogManager.h"
#include "GameManager.h"
#include "GameObject.h"
#include "Position.h"
#include <iostream>
#include <unistd.h>
#include <string>
#include "GameObjectList.h"
#include "GameObjectListIterator.h"
#include "Event.h"
#include "EventStep.h"
#include "WorldManager.h"
#include "Manager.h"
#include "InputManager.h"
#include "EventMouse.h"
#include "EventKeyboard.h"
#include "EventCollision.h"
#include "EventOut.h"
#include "GraphicsManager.h"
using namespace std;

WorldManager* WorldManager::singletonInstance = NULL;


WorldManager::WorldManager()
{
	
}

WorldManager::~WorldManager()
{
   
}

WorldManager* WorldManager::getInstance()
{
    if (singletonInstance == NULL)
    {
        singletonInstance = new WorldManager();
    }
    return singletonInstance;
}

GameObjectList WorldManager::getAllObjects()
{
    return obj;
}

int WorldManager::insertObj(GameObject *p_go)
{
    if(obj.insert(p_go) == 0)
    {
        LogManager *logManager = LogManager::getInstance();
        logManager->writeLog("OBJECT INSERTED");
        return 0;
    }
    return -1;
}

int WorldManager::removeObj(GameObject *p_go)
{
    if(obj.remove(p_go) == 0)
    {
        return 0;
    }
    return -1;
}

GameObject * WorldManager::findObj(int id)
{
    return obj.find(id);
}

void WorldManager::update()
{
     Manager manager = Manager::Manager();
     EventStep step;
     manager.onEvent(&step);
	 
	 // Update object positions based on their velocities
	 GameObjectListIterator vi(&obj);
     vi.first();
	 while(vi.isDone() == false)
     {
		 GameObject *p_go = vi.currentObj();
		 float x = p_go->getXVelocityStep();  // see how far moved x
		 float y = p_go->getYVelocityStep();  // see how far moved y

		 if(x != 0.00000)
		 {
			Position new_pos(p_go->getPos().getX() + (int)x, p_go->getPos().getY() + (int)y);
			moveObj(p_go, new_pos);

		 }
		 vi.next();
     }



     GameObjectListIterator goli(&del);
     goli.first();
	 while(goli.isDone() == false)
     {
       delete(goli.currentObj());
	  // delete(goli.currentObj()->getSprite());
       goli.next();
	   LogManager *logManager = LogManager::getInstance();
	   logManager->writeLog("Deleting Object");
     }
	  
     del.clear();
}

int WorldManager::startUp(bool append, bool flush)
{
    return 1;
}

void WorldManager::draw()
{
    GameObjectListIterator goli(&obj);
    for(int i = -2; i <= 2; i++)
    {
         goli.first();
         while(goli.isDone() == false)
         {
             if(goli.currentObj()->getAltitude() == i)
             {
				
				Box box = goli.currentObj()->getBox();
				Position corner = box.getCorner();
				corner.setX(corner.getX() + goli.currentObj()->getPos().getX());
				corner.setY(corner.getY() + goli.currentObj()->getPos().getY());
				box.setCorner(corner);
				goli.currentObj()->draw();
             }
             goli.next();
         }
    }
}

bool WorldManager::boxesIntersect(Position b1, Position b2, GameObject *p_go)
{
    if(xInt(b1,b2,p_go) == 1 && yInt(b1,b2,p_go) == 1)
    {
		return true;
    }
  return false;
}

bool WorldManager::xInt(Position b1, Position b2, GameObject *p_go)
{
	if(((b2.getX() <= b1.getX()) && (b1.getX() <= (b2.getX())))
		|| ((b1.getX() <= b2.getX()) && (b2.getX() <= (b1.getX() + p_go->getBox().getHorizontal()))))
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool WorldManager::yInt(Position b1, Position b2, GameObject *p_go)
{
	if(((b2.getY() <= b1.getY()) && (b1.getY() <= (b2.getY())))
		|| ((b1.getY() <= b2.getY()) && (b2.getY() <= (b1.getY() + p_go->getBox().getVertical()))))
	{
		return true;
	}
	else
	{
		return false;
	}

}

GameObject* WorldManager::isCollision(GameObject *p_go, Position where)
{
     GameObjectListIterator goli(&obj);
     goli.first();
     while(goli.isDone() == false)
     {
         GameObject *p_temp_go = goli.currentObj();
         if(p_temp_go != p_go) //then not self
         {
            if (boxesIntersect(p_temp_go -> getPos(), where, p_temp_go))
            {
                if(p_temp_go -> isSolid())
                {
                    return p_temp_go;
                }
            }
         }
         goli.next();
     }
    return NULL; // if here, no collision
}

int WorldManager::moveObj(GameObject* p_go, Position where)
{
    GameObject *p_temp_go;
    GraphicsManager *graphicsManager = GraphicsManager::getInstance();
    p_temp_go = isCollision(p_go, where);  // collide?  Null if not

    if(p_temp_go)
    {
		if(p_go->getAltitude() == 0 || p_temp_go->getAltitude() == 0)
		{
			if((p_temp_go->getAltitude() == 1 || p_temp_go->getAltitude() == 2) || (p_go->getAltitude() == 1 || p_go->getAltitude() == 2))
			{
				EventCollision c(p_go, p_temp_go, where);  // create event
				p_go -> eventHandler(&c);                   // send to obj
				p_temp_go -> eventHandler(&c);             // send to other
				return 0;
			}
		}
    }
    p_go -> setPos(where); // if here, no collision so allow move
	p_go -> getBox().setCorner(where);

	if (p_view_following == p_go)
	{
		setViewPosition(p_go->getPos());
	}

    if((where.getX() > graphicsManager->getHorizontal() || where.getX() < 0 ||
        where.getY() > graphicsManager->getVertical() || where.getY() < 0))
        {   
            EventOut ov;
            p_go -> eventHandler(&ov);
        }
    return 0;
}

int WorldManager::markForDelete(GameObject * p_go)
{

	GameObjectListIterator goli(&del);
     goli.first();
     while(goli.isDone() == false)
     {
         if(goli.currentObj() == p_go)
         {
			 return 0;
         }
         goli.next();
     }
    
	 del.insert(p_go);
     return 0;
}


Box WorldManager::getBoundary()
{
	return boundary;
}

void WorldManager::setBoundary(Box new_boundary)
{
	boundary = new_boundary;
}

Box WorldManager::getView()
{
	return view;
}

void WorldManager::setView (Box new_view)
{
	view = new_view;
}

int WorldManager::setViewFollowing(GameObject *p_new_view_following)
{

	if(p_new_view_following == NULL)
	{
		p_view_following = NULL;
		return 0;
	}
// Iterate over all objects, make sure new one legitimate

	 GameObjectListIterator goli(&obj);
     goli.first();
     while(goli.isDone() == false)
     {
         if(goli.currentObj() == p_new_view_following)
         {
			 p_view_following = p_new_view_following;
			 setViewPosition(p_view_following -> getPos());
			 return 0;
         }
         goli.next();
     }
    return -1;
}

void WorldManager::setViewPosition(Position view_pos)
{
	// make sure horizontal not out of world boundaries
int x = view_pos.getX() - view.getHorizontal()/2;
if (x + view.getHorizontal() > boundary.getHorizontal())
{
	x = boundary.getHorizontal()-view.getHorizontal();
}
if (x < 0)  // limit range to stay within world boundary
{
  x = 0;
}

// make sure vertical not out of world boundaries
int y = view_pos.getY() - view.getVertical()/2;

if (y + view.getVertical() > boundary.getVertical())
{
	y = boundary.getVertical()-view.getVertical();
}
if (y < 0)  // limit range to stay within world boundary
{
  y = 0;
}

// set view
Position new_corner(x, y); 
view.setCorner(new_corner);
}

int WorldManager::shutDown()
{
    return 1;
}
