//
// WorldManager.h
//

#ifndef __GAME_WORLD_H__
#define __GAME_WORLD_H__

#include "GameObject.h"
#include "GameObjectListIterator.h"
#include "GameObjectList.h"
#include "Position.h"
#include "Manager.h"
#include "Box.h"

class WorldManager : public Manager {

 protected:
  int obj_id;			       // next object id to assign
  WorldManager (WorldManager const&);  // don't allow copy
  void operator=(WorldManager const&); // don't allow assignment
  WorldManager();		       // private since a singleton
  GameObjectList obj;		       // list of all game objects
  GameObjectList del;			//list of all objects to be deleted
  static WorldManager* singletonInstance;
  Box boundary;
  Box view;
  GameObject *p_view_following;


 public:
  ~WorldManager();
  static WorldManager *getInstance();

  // returns a list of all objects (obj)
  GameObjectList getAllObjects(void);

  // add object to game world (obj)
  int insertObj(GameObject *p_go);

  // remove object to game world (obj)
  int removeObj(GameObject *p_go);
  
  GameObject *findObj(int id);
  
  void update();

  void draw();

  bool boxesIntersect(Position b1, Position b2, GameObject *p_go);

  GameObject* isCollision(GameObject *p_go, Position where);

  int moveObj(GameObject* p_go, Position where);

  int startUp(bool append, bool flush);

  int shutDown();

  int markForDelete(GameObject * p_go);

  bool xInt(Position b1, Position b2, GameObject *p_go);

  bool yInt(Position b1, Position b2, GameObject *p_go);

  Box getBoundary();

  void setBoundary(Box new_boundary);

  Box getView();

  void setView (Box new_view);

  int setViewFollowing(GameObject *p_new_view_following);

  void setViewPosition(Position view_pos);

  



};

#endif // __GAME_WORLD_H__

